/*
	Stock Quote Applet
	Internet Programming with Java Course
	
	(c) 2002 Martin Valkanov
	
	The applet is a stock quote monitor, 
	that offers a real-time chart.
	
	The data is read from the server using 
	the StockQuoteListener.
	
	The display part is implemented in StockQuotePanel.
*/

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class StockQuoteApplet extends Applet implements ActionListener
{
	private static final int TICKER_VALUES_CNT = 50;
	
	Label labelTicker;
	TextField textFieldTicker;
	Button btnShowChart;
	Panel panelGraphic;
	
	double lfTickerValues[];
	int nTimeStart[];
	StockQuoteListener quoteListener = null;
	
	int nTickerColorIndex[];
	
	public void init()
	{
		String serverHost = getParameter( "serverHost" );
		String serverPort = getParameter( "serverPort" );
		
		if ( ( serverHost != null ) && ( serverPort != null ) )
		{
			try
			{
				int port = new Integer( serverPort ).intValue();
				
				StockQuoteListener.serverHost = serverHost;
				StockQuoteListener.serverPort = port;
			}
			catch ( NumberFormatException exNumberFormat )
			{
			}
		}
		
		Container cont = this;
		cont.setLayout( new BorderLayout() );
		
		Panel panelTop = new Panel();
			
		labelTicker = new Label( "Ticker:" );
		panelTop.add( labelTicker );
		
		textFieldTicker = new TextField(4);
		panelTop.add( textFieldTicker );
		
		panelTop.setSize( new Dimension( panelTop.getMaximumSize().width, panelTop.getMinimumSize().height ) );
		
		btnShowChart = new Button( "Show chart" );
		panelTop.add( btnShowChart );
		
		cont.add( panelTop, BorderLayout.NORTH );
		
		lfTickerValues = new double[TICKER_VALUES_CNT];
		for ( int i = 0; i < lfTickerValues.length; i++ ) lfTickerValues[i] = -1;
		
		nTickerColorIndex = new int[TICKER_VALUES_CNT];
		for ( int i = 0; i < nTickerColorIndex.length; i++ ) nTickerColorIndex[i] = i;
		
		nTimeStart = new int[1];
		nTimeStart[0] = 0;
		
		panelGraphic = new StockQuotePanel( lfTickerValues, nTimeStart, nTickerColorIndex );
		
		Panel panelBottom = new Panel();
		panelBottom.setLayout( new GridLayout( 1, 1, 5, 5 ) );
		cont.add( panelBottom, BorderLayout.CENTER );
		
		panelBottom.add( panelGraphic );
		
		btnShowChart.addActionListener(this);
	}
	
	public void destroy()
	{
		// Terminate the listening thread and clean-up data values.
		if ( quoteListener != null )
		{
			quoteListener.terminate();
			quoteListener = null;
		}
		
		Container cont = this;
		cont.removeAll();
		
		super.destroy();
	}
	
	// Show chart button pressed
	public void actionPerformed( ActionEvent e )
	{
		if ( e.getSource() == btnShowChart )
		{
			// Terminate the previous listening thread and clean-up data values.
			if ( quoteListener != null )
			{
				quoteListener.terminate();
				quoteListener = null;
			}
			
			for ( int i = 0; i < lfTickerValues.length; i++ ) lfTickerValues[i] = -1;
			
			// Ticker id has to be 4-letter sequence.
			String sTickerId = textFieldTicker.getText();
			
			if ( sTickerId != sTickerId.toUpperCase() )
			{
				sTickerId = sTickerId.toUpperCase();
				textFieldTicker.setText( sTickerId );
			}
			
			boolean bNotLatin = false;
			
			for ( int i = 0; i < sTickerId.length(); i++ )
				if ( ( sTickerId.charAt(i) < 'A' ) || ( sTickerId.charAt(i) > 'Z' ) )
				{
					bNotLatin = true;
					break;
				}
			
			if ( ( sTickerId.length() != 4 ) || bNotLatin )
			{
				MessageBox msgBox = new MessageBox( "Syntax error", "Please enter a 4-latin-letter sequence." );
				msgBox.setVisible( true );
				return;
			}
			
			// Startup a new data monitoring thread.
			quoteListener = new StockQuoteListener( lfTickerValues, nTimeStart, sTickerId, panelGraphic, nTickerColorIndex );
			quoteListener.start();
		}
	}
}