import java.io.*;
import java.net.*;
import java.awt.*;

class StockQuoteListener extends Thread
{
	public final static String SERVER_HOST = "localhost";
	public final static int SERVER_PORT = 3003;
	private final int QUERY_INTERVAL = 500;
	
	public static String serverHost = SERVER_HOST;
	public static int serverPort = SERVER_PORT;
	
	double lfTickerValues[];
	int nTimeStart[];
	String tickerId;
	boolean bNotTerminated = true;
	
	int nTickerColorIndex[];
	
	Component repaintComp;
	
	public StockQuoteListener( double lfTickerValues[], int nTimeStart[], String tickerId, Component repaintComp, int nTickerColorIndex[] )
	{
		this.lfTickerValues = lfTickerValues;
		this.nTimeStart = nTimeStart;
		this.tickerId = tickerId;
		this.repaintComp = repaintComp;
		this.nTickerColorIndex = nTickerColorIndex;
	}
	
	public void terminate()
	{
		bNotTerminated = false;
	}
	
	public void run()
	{
		Socket sock;
		
		try
		{
			try
			{
				sock = new Socket( serverHost, serverPort );
			}
			catch ( UnknownHostException exUnknownHost )
			{
				MessageBox msgBox = new MessageBox( "Communication error", "Unknown server host \"" + SERVER_HOST + "\"." );
				msgBox.show();
				
				System.err.println( "Unknown server host \"" + SERVER_HOST + "\"" );
				return;
			}
		}
		catch ( IOException exIO )
		{
			MessageBox msgBox = new MessageBox( "Communication error", "Unable to establish socket connection to server." );
			msgBox.show();
			
			System.err.println( "Unable to establish socket connection to server." );
			return;
		}
		
		try
		{
			try
			{
				BufferedReader in = new BufferedReader( new InputStreamReader( sock.getInputStream() ) );
				PrintWriter out = new PrintWriter( new OutputStreamWriter( sock.getOutputStream() ) );				
				
				out.println( tickerId );
				out.flush();
				
				String s;
				double lfValue;
				int i, iw = 0;
				
				while ( bNotTerminated )
				{
					try
					{
						Thread.sleep( QUERY_INTERVAL );
					}
					catch ( InterruptedException exIntr )
					{
						terminate();
						break;
					}
					
					if ( ( s = in.readLine() ) == null ) break;
					lfValue = ( new Double(s) ).doubleValue();
					
					if ( iw < lfTickerValues.length )
						lfTickerValues[iw++] = lfValue;
					else
					{
						int nColorIndex = nTickerColorIndex[0];
						
						for ( i = 1; i < iw; i++ )
						{
							lfTickerValues[i - 1] = lfTickerValues[i];
							nTickerColorIndex[i - 1] = nTickerColorIndex[i];
							nColorIndex = nTickerColorIndex[i - 1];
						}
							
						lfTickerValues[iw - 1] = lfValue;
						nTickerColorIndex[iw - 1] = nColorIndex + 1;
						
						++nTimeStart[0];
					}
					
					//repaintComp.repaint();
					Graphics g = repaintComp.getGraphics();
					if ( g != null )
						repaintComp.paint( g );
				}
			}
			catch ( IOException exIO )
			{
				MessageBox msgBox = new MessageBox( "Communication error", "Socket communication failed." );
				msgBox.show();
				
				System.err.println( "Socket communication failed." );
			}
		}
		finally
		{
			try
			{
				sock.close();
			}
			catch ( IOException exIO )
			{
			}
		}
	}
}