/*
	Stock Quote Applet
	Internet Programming with Java Course
	
	(c) 2002 Martin Valkanov
		
	StockQuotePanel implements the display part.
	
	The applet passes a reference to the data array,
	values in which will later be changed by the StockQuoteListener.
	
	Data is displayed via drawing polygons with 4 vertices, filling
	the area between every two consequtive moments 
*/

import java.awt.*;

class StockQuotePanel extends Panel
{
	double lfTickerValues[];
	int nTimeStart[];
	Image bufImage;
	Rectangle bufBounds = new Rectangle( 0, 0, 0, 0 );
	
	int nTickerColorIndex[];
	public static Color vTickerColors[] = { Color.white, Color.blue, Color.black, Color.cyan, Color.orange, Color.red };
	
	public StockQuotePanel( double lfTickerValues[], int nTimeStart[], int nTickerColorIndex[] )
	{
		this.lfTickerValues = lfTickerValues;
		this.nTimeStart = nTimeStart;
		this.nTickerColorIndex = nTickerColorIndex;
	}
	
	public void paint( Graphics g )
	{
		Rectangle rect = getBounds();
		
		if ( !bufBounds.equals( rect ) )
		{
			bufBounds = rect;
			bufImage = createImage( bufBounds.width, bufBounds.height );
		}
		
		Graphics oldg = g;
		g = bufImage.getGraphics();
		
		rect.translate( -rect.x, -rect.y );
		
		// Erase background
		g.setColor( Color.white );
		g.fillRect( rect.x, rect.y, rect.width, rect.height );
		
		// Determine where to display the labels on the left
		g.setColor( Color.black );
		
		int vVertLevels[] = new int[11];
		int nLevelHeight = rect.height / ( vVertLevels.length + 1 );
		// "+1" to provide space for an additional timeline
		
		int i;
		int nSpaceWidth = g.getFontMetrics( g.getFont() ).charWidth( '0' );
		int nDigitHeight = g.getFontMetrics( g.getFont() ).getAscent();
		int nLeftOffset = g.getFontMetrics( g.getFont() ).stringWidth( "1000,00" );
		
		nLeftOffset += nSpaceWidth * 2;
		
		vVertLevels[0] = rect.y + ( nLevelHeight / 2 );
		for ( i = 1; i < vVertLevels.length; i++ )
			vVertLevels[i] = vVertLevels[0] + i*nLevelHeight;
			
		// Print the value labels
		for ( i = 0; i < vVertLevels.length; i++ )
		{
			String s = (vVertLevels.length - 1 - i)*100 + ",00";
			g.drawString( s, rect.x + nSpaceWidth*( "1000,00".length() - s.length() + 1 ), vVertLevels[i] + ( nDigitHeight / 2 ) );
			g.drawLine( rect.x + nLeftOffset, vVertLevels[i], rect.x + nLeftOffset + nSpaceWidth, vVertLevels[i] );
		}
		
		int nBottomLine = vVertLevels[ vVertLevels.length - 1 ];
		int nRightLine = rect.x + rect.width - nSpaceWidth * 2;
		
		// Calculate the field where to display data 
		int nFieldWidth, nFieldHeight, nFieldLeft, nFieldTop;
		
		nFieldLeft = rect.x + nLeftOffset + nSpaceWidth; 
		nFieldTop = vVertLevels[0];
		
		nFieldWidth = nRightLine - nFieldLeft + 1;
		nFieldHeight = nBottomLine - nFieldTop;

		// Draw a grid on the background
		g.drawLine( nFieldLeft, vVertLevels[0], nRightLine, vVertLevels[0] );
		
		for ( i = 1; i < vVertLevels.length; i++ )
		{			
			g.setColor( Color.lightGray );
			g.fillRect( nFieldLeft, vVertLevels[i - 1] + 1, nFieldWidth, nLevelHeight - 1 );
			
			g.setColor( Color.black );
			g.drawLine( nFieldLeft, vVertLevels[i], nFieldLeft + nFieldWidth - 1, vVertLevels[i] );
		}
		
		double nHorizMarkSpace = ((double) nFieldWidth) / ( lfTickerValues.length - 1 );
		
		for ( i = 1; ( i < lfTickerValues.length ) && ( lfTickerValues[i] != -1 ); i++ )
		{
			double mv = lfTickerValues[i - 1];
			if ( lfTickerValues[i] < mv ) mv = lfTickerValues[i];
			
			Polygon p = new Polygon();
			
			p.addPoint( nFieldLeft + (int) ( (i - 1)*nHorizMarkSpace ),
						nFieldTop + (int) ( nFieldHeight*(1 - 0.001*lfTickerValues[i - 1]) ) );
						
			p.addPoint( nFieldLeft + (int) ( i*nHorizMarkSpace ),
						nFieldTop + (int) ( nFieldHeight*(1 - 0.001*lfTickerValues[i]) ) );
						
			p.addPoint( nFieldLeft + (int) ( i*nHorizMarkSpace ),
						nFieldTop + nFieldHeight );
						
			p.addPoint( nFieldLeft + (int) ( (i - 1)*nHorizMarkSpace ),
						nFieldTop + nFieldHeight );
			
			//g.setColor( Color.blue );
			g.setColor( vTickerColors[ (nTickerColorIndex[i - 1] / 10) % vTickerColors.length ] );
			g.fillPolygon(p);
	
			// Draw the actual data line		
			g.setColor( Color.black );
			g.drawLine( nFieldLeft + (int) ( (i - 1)*nHorizMarkSpace ),
						nFieldTop + (int) ( nFieldHeight*(1 - 0.001*lfTickerValues[i - 1]) ),
						nFieldLeft + (int) ( i*nHorizMarkSpace ),
						nFieldTop + (int) ( nFieldHeight*(1 - 0.001*lfTickerValues[i]) ) );
		}
		
		// Draw the grid on the bottom
		for ( i = 0; i < lfTickerValues.length - 1; i++ )
			g.drawLine( nFieldLeft + (int) ( i*nHorizMarkSpace ), nFieldTop + nFieldHeight,
						nFieldLeft + (int) ( i*nHorizMarkSpace ), nFieldTop + nFieldHeight + ( nLevelHeight / 10 ) );
						
		g.drawLine( nFieldLeft, nFieldTop, nFieldLeft, nFieldTop + nFieldHeight );
		g.drawLine( nFieldLeft + nFieldWidth - 1, nFieldTop, nFieldLeft + nFieldWidth - 1, nFieldTop + nFieldHeight );
		
		FontMetrics fm = g.getFontMetrics( g.getFont() );
		int nTimeLabelsTop = nFieldTop + nFieldHeight + 15 + ( nLevelHeight / 2 );
		String s;
		
		int nTimeSeg, nTimeOfs;
		
		nTimeSeg = ( nTimeStart[0] / lfTickerValues.length ) * lfTickerValues.length;
		nTimeOfs = nTimeStart[0] - nTimeSeg;
		
		s = "Time: " + nTimeSeg;
		
		g.drawString( s, ( rect.x + nFieldLeft ) / 2 - fm.stringWidth(s) / 2, nTimeLabelsTop );
		
		for ( i = 0; i < lfTickerValues.length - 1; i++ ) if ( (nTimeOfs + i) % 10 == 0 )
		{
			g.drawLine( nFieldLeft + (int) ( i*nHorizMarkSpace ), nFieldTop + nFieldHeight,
						nFieldLeft + (int) ( i*nHorizMarkSpace ), nFieldTop + nFieldHeight + ( nLevelHeight / 5 ) );
			
			s = "+" + ( nTimeOfs + i );
			g.drawString( s, nFieldLeft + (int) ( i*nHorizMarkSpace ) - fm.stringWidth(s) / 2, nTimeLabelsTop );
		}
		
		oldg.drawImage( bufImage, 0, 0, null );
	}
	
	public void paintComponents( Graphics g )
	{
	}
}