package nick.inetjava.pop3applet;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * <p>
 * The applet runs in 4 threads:
 * <ul><li>The main thread which initializes the applet and builds the GUI
 *     <li>The event-dispatching thread of the GUI
 *     <li>A POP3Thread, which sends and receives POP3 commands
 *     <li>A ConnectionPreservingThread, responsible of sending a 'stat' command every 20 seconds
 * <p>
 * The communication between the POP3Thread and the other threads is via a synchronized waiting queue,
 * i.e. a queue which blocks execution when there is no data but pop() is invoked until data is available.
 * Instances of POP3Command are pushed into the queue.
 *
 * @author Nick Nickolov, nick_nickolov@abv.bg
 * @version May 20, 2002 9:07:33 PM
 */
public final class POP3Applet extends Applet {
  static final boolean DEBUG = false;
  static final boolean USE_NAKOV_TCP_PROXY = true;
  static final String NAKOV_TCP_PROXY_HOST = "localhost";
  static final int    NAKOV_TCP_PROXY_PORT = 3004;
  private static final Color TEXT_FIELD_BG_COLOR = new Color(0x00, 0xFF, 0xFF);

  private TextField tfServer;
  private TextField tfLogin;
  private TextField tfPassword;
  private TextArea textArea;
  private List list;
  private TextField tfStatus;
  private Label messageInfoLabel;

  private String[] messageList;
  private Queue queue;


  public POP3Applet() {
    queue = new Queue();
    POP3Thread pop3Thread = new POP3Thread(this, queue);
    pop3Thread.start();
    ConnectionPreservingThread connectionPreservingThread
            = new ConnectionPreservingThread(pop3Thread, queue);
    connectionPreservingThread.start();
  }

  public void init() {
    super.setLayout(new BorderLayout(5, 5));
    Panel topPanel = new Panel(new BorderLayout(5, 5));
    // Creating the topmost fields
    Panel inputPanel = new Panel(new FlowLayout(FlowLayout.CENTER, 0, 0));
    inputPanel.add(new Label("POP3 Server:"));
    tfServer = new TextField(20);
    inputPanel.add(tfServer);
    inputPanel.add(new Label("   Login:"));
    tfLogin = new TextField(25);
    inputPanel.add(tfLogin);
    inputPanel.add(new Label("   Password:"));
    tfPassword = new TextField(10);
    tfPassword.setEchoChar('*');
    inputPanel.add(tfPassword);
    tfServer.setBackground(TEXT_FIELD_BG_COLOR);
    tfLogin.setBackground(TEXT_FIELD_BG_COLOR);
    tfPassword.setBackground(TEXT_FIELD_BG_COLOR);
    topPanel.add(inputPanel, BorderLayout.NORTH);
    // Creating the buttons
    Panel controlsPanel = new Panel(new FlowLayout(FlowLayout.RIGHT));
    Button buttonCheckMail = new Button("Check Mail");
    buttonCheckMail.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        queue.push(new POP3Command(POP3Command.C_CHECK_MAIL,
                                   new Object[] { tfServer.getText(), tfLogin.getText(), tfPassword.getText(), }));
      }
    });
    controlsPanel.add(buttonCheckMail);
    Button buttonDisconnect = new Button("Disconnect");
    buttonDisconnect.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        queue.push(new POP3Command(POP3Command.C_DISCONNECT, null));
      }
    });
    controlsPanel.add(buttonDisconnect);
    topPanel.add(controlsPanel, BorderLayout.EAST);
    Panel messageInfoPanel = new Panel(new BorderLayout(0, 0));
    messageInfoLabel = new Label();
    messageInfoPanel.add(messageInfoLabel, BorderLayout.SOUTH);
    topPanel.add(messageInfoPanel, BorderLayout.WEST);
    super.add(topPanel, BorderLayout.NORTH);
    // Creating the text area and the list
    textArea = new TextArea();
    textArea.setEditable(false);
    super.add(textArea, BorderLayout.CENTER);
    list = new List();
    final PopupMenu popupMenu = new PopupMenu();
    MenuItem mi;
    mi = new MenuItem("Show Message");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int index = list.getSelectedIndex();
        if (index == -1) {
          return;
        }
        String s = list.getItem(index);
        try {
          s = s.substring(s.indexOf(' ') + 1);
        } catch (Exception ex) {
          s = "???";
        }
        messageInfoLabel.setText("Message shown: " + index + " (" + s + " bytes)");
        queue.push(new POP3Command(POP3Command.C_SHOW_MESSAGE, new Object[] { new Integer(index), }));
      }
    });
    popupMenu.add(mi);
    mi = new MenuItem("Delete Message");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int index = list.getSelectedIndex();
        if (index == -1) {
          return;
        }
        queue.push(new POP3Command(POP3Command.C_DELETE_MESSAGE, new Object[] { new Integer(index), }));
      }
    });
    popupMenu.add(mi);
    list.add(popupMenu);
    list.addMouseListener(new MouseListener() {
      public void mouseClicked(MouseEvent e) { }
      public void mousePressed(MouseEvent e) {
        if ((e.getModifiers() & 4) != 0) {
          popupMenu.show(list, e.getX(), e.getY());
        }
      }
      public void mouseReleased(MouseEvent e) { }
      public void mouseEntered(MouseEvent e) { }
      public void mouseExited(MouseEvent e) { }
    });
    ScrollPane listScrollPane = new ScrollPane();
    listScrollPane.add(list);
    super.add(listScrollPane, BorderLayout.EAST);
    // Creating the status field
    tfStatus = new TextField();
    tfStatus.setEnabled(false);
    tfStatus.setEditable(false);
    super.add(tfStatus, BorderLayout.SOUTH);
    // debug
    if (DEBUG) {
      tfServer.setText("apop.zzn.com");
      tfLogin.setText("nick_nickolov@sethi.zzn.com");
      tfPassword.setText("string");
      list.add("Aneliq");
      list.add("Bilqna");
      list.add("Cwetanka");
      list.add("Dimitrina");
      list.add("Elena");
      textArea.setText("Applets are a piece of bullshit.\nUse swing instead.");
    }
  }

  /**
   * Invoked on the POP3Thread
   */
  void commandCompleted(POP3Command command, boolean successful, Object result) {
    if (DEBUG) {
      System.out.println("DUMP commandCompleted command=" + command.getCommand() + " successful=" + successful + " result=" + result);
    }
    if (!successful) {
      setStatus((String) result);
      return;
    }
    switch (command.getCommand()) {
      case POP3Command.C_CHECK_MAIL: {
        String[] r = (String[]) result;
        messageList = r;
        list.removeAll();
        for (int i = 0; i < r.length; i++) {
          list.add(r[i]);
        }
        break;
      }
      case POP3Command.C_SHOW_MESSAGE: {
        String r = (String) result;
        r = (r == null) ? "" : r;
        textArea.setText(r);
        break;
      }
      default: {
        // ignore
        break;
      }
    }
  }

  /**
   * Invoked on the POP3Thread
   */
  void setStatus(String s) {
    tfStatus.setText(s);
  }
}
