package nick.inetjava.pop3applet;

/**
 * A synchronized 'waiting' queue.
 * When pop() is invoked but the queue is empty, execution is
 * suspended until data is available.
 *
 * @author Nick Nickolov, nick_nickolov@abv.bg
 * @version Jun 1, 2002 4:42:09 PM
 */
final class Queue {
  private Object[] data = new Object[32];
  private int nData;
  private Object lock = new Object(); // notified when new data is pushed

  void push(Object x) {
    synchronized (lock) {
      if (nData >= data.length) {
        Object[] old = data;
        data = new Object[nData * 2];
        System.arraycopy(old, 0, data, 0, nData);
      }
      data[nData] = x;
      nData++;
      lock.notify();
    }
  }

  Object pop() {
    synchronized (lock) {
      if (nData == 0) {
        try {
          lock.wait();
        } catch (InterruptedException e) {
        }
      }
      Object r = data[0];
      nData--;
      System.arraycopy(data, 1, data, 0, nData);
      return r;
    }
  }

  void clear() {
    synchronized (lock) {
      nData = 0;
    }
  }
}
