package nick.inetjava.server;

import java.util.*;
import java.io.*;

/**
 * @author
 * @version 1.0
 */

final class VirtualFileSystem {
  private static final String[] STRING_ARRAY_0 = new String[0];

  private SimpleWebServer owner;
  private String[][] virtualRoots;
  private File[] physicalRoots;

  private Vector temp = new Vector();
  private Vector temp2 = new Vector();
  private Vector temp3 = new Vector();


  protected VirtualFileSystem(SimpleWebServer owner) {
    this.owner = owner;
    temp.clear();
    temp2.clear();
    Properties conf = owner.getConf();
    for (Enumeration enum = conf.keys(); enum.hasMoreElements(); ) {
      String key = (String) enum.nextElement();
      if (key.equals("root") || key.startsWith("root.")) {
        String[] path = (key.equals("root")) ? STRING_ARRAY_0 : parsePath(key.substring(5));
        String value = (String) conf.get(key);
        owner.log("Associating: " + key + " --> " + value);
        temp.add(path);
        temp2.add(new File(value));
      }
    }
    virtualRoots = (String[][]) temp.toArray(new String[0][0]);
    physicalRoots = (File[]) temp2.toArray(new File[0]);
    for (int i = 0; i < physicalRoots.length; i++) {
      try {
        physicalRoots[i] = physicalRoots[i].getCanonicalFile();
      } catch (IOException ioe) {
        // ignore
      }
    }
  }

  protected VirtualFile get(String sPath) {
    String[] path = parsePath(sPath);
    owner.log("vfs.get(" + toString(path) + ")");
    if (path == null) {
      return null;
    }
    for (int i = 0; i < virtualRoots.length; i++) {

      if (isPrefix(virtualRoots[i], path)) {
        owner.log("requested file found on virtual root '" + toString(virtualRoots[i]) + "' -> " + physicalRoots[i]);
        return createVirtualFile(i, path);
      } else if (isPrefix(path, virtualRoots[i])) {
        owner.log("requested file found as a subpath of virtual root '" + toString(virtualRoots[i]) + "'");
        return createImaginaryVirtualFile(path);
      }
      owner.log("requested file has nothing to do with virtual root '" + toString(virtualRoots[i]) + "'");
    }
    return null;
  }

  private VirtualFile createVirtualFile(int i, String[] path) {
    owner.log("dump path = " + toString(path));
    owner.log("dump virtualRoots[i] = " + toString(virtualRoots[i]));
    String filename = (path.length == virtualRoots[i].length) ? "." : toString(path, virtualRoots[i].length);
    File f = new File(physicalRoots[i], filename);
    owner.log("dump f = " + f.getAbsoluteFile());
    if (!f.exists() || !f.canRead()) {
      return null;
    }
    if (f.isDirectory()) {
      return new VirtualFile(path, f, f.list());
    } else {
      return new VirtualFile(path, f, null);
    }
  }

  private VirtualFile createImaginaryVirtualFile(String[] path) {
    temp.clear();
    for (int i = 0; i < virtualRoots.length; i++) {
      if (isPrefix(path, virtualRoots[i])
                  && (path.length < virtualRoots[i].length)
                  && !temp.contains(virtualRoots[i][path.length])) {
        temp.add(virtualRoots[i][path.length]);
      }
    }
    return new VirtualFile(path, null, (String[]) temp.toArray(STRING_ARRAY_0));
  }

  /**
   * Tokenizes and canonicalizes the abstract path
   */
  private String[] parsePath(String path) {
    temp3.clear();
    int lPath = path.length();
    path = path.replace('\\', '/');
    int i = 0;
    while (i < lPath) {
      int j = i;
      while ((j < lPath) && (path.charAt(j) != '/')) {
        j++;
      }
      if (i < j) {
        String token = path.substring(i, j);
        if (!token.equals(".")) {
          if (token.equals("..")) {
            if (temp3.size() == 0) {
              return null;
            } else {
              temp3.setSize(temp3.size() - 1);
            }
          } else {
            temp3.add(token);
          }
        }
      }
      i = j + 1;
    }
    return (String[]) temp3.toArray(STRING_ARRAY_0);
  }

  private boolean isPrefix(String[] prefixPath, String[] path) {
    int lPrefixPath = prefixPath.length;
    int lPath = path.length;
    if (lPrefixPath > lPath) {
      return false;
    }
    for (int i = 0; i < lPrefixPath; i++) {
      if (!prefixPath[i].equals(path[i])) {
        return false;
      }
    }
    return true;
  }

  protected static String toString(String[] path, int fromIndex) {
    if (path == null) {
      return "/";
    }
    StringBuffer b = new StringBuffer();
    if (fromIndex < path.length) {
      b.append(path[fromIndex]);
    }
    for (int i = fromIndex + 1; i < path.length; i++) {
      b.append('/').append(path[i]);
    }
    return b.toString();
  }

  protected static String toString(String[] path) {
    return toString(path, 0);
  }
}
