import java.io.*;
import java.net.*;
import java.awt.*;
import java.applet.*;
import java.awt.event.*;

public class MailReader extends Applet implements ActionListener, ItemListener
{
	private TextField server, username, password;
	private Button connect, message_delete/*, message_refresh*/;
	private Panel logon_panel, message_panel;
	private List message_list;
	private Checkbox message_verbose;
	private TextArea message_text;
	private Label status;
	private boolean connected = false;

	private final int PROXY_PORT = 3004;
	private final int POP3_PORT = 110;
	private Socket socket;
	private PrintStream output;
	private BufferedReader input;

	private KeepAlive alive;
	private final int KEEPALIVE_INTERVAL = 20000;

	public void init()
	{
		// initialize GUI
		setFont( new Font( "Courier", Font.PLAIN, 12 ) );
		server = new TextField( 20 );
		username = new TextField( 10 );
		password = new TextField( 10 );
		password.setEchoChar( '*' );
		connect = new Button( "Check mail" );
		connect.addActionListener( this );
		logon_panel = new Panel( new GridBagLayout() );
		message_panel = new Panel( new GridBagLayout() );
		message_panel.setEnabled( false );
		message_list = new List( 10, false );
		message_list.addItemListener( this );
		message_verbose = new Checkbox( "Verbose list" );
		message_verbose.addItemListener( this );
		message_text = new TextArea( 10, 35 );
		message_text.setEditable( false );
		message_delete = new Button( "Delete message" );
		message_delete.addActionListener( this );
//		message_refresh = new Button( "Refresh" );
//		message_refresh.addActionListener( this );
		status = new Label();
		status.setFont( new Font( "Courier", Font.ITALIC, 14 ) );
		setStatus( false, "Not connected" );

		GridBagConstraints c1 = new GridBagConstraints();
		c1.gridwidth = 1;
		c1.gridheight = 1;
		c1.weightx = 1.0;
		c1.weighty = 1.0;
		c1.anchor = GridBagConstraints.WEST;
		GridBagConstraints c2 = (GridBagConstraints)c1.clone();
		c2.gridwidth = GridBagConstraints.REMAINDER;
		logon_panel.add( new Label( "Server", Label.LEFT ), c1 );
		logon_panel.add( server, c2 );
		logon_panel.add( new Label( "Username", Label.LEFT ), c1 );
		logon_panel.add( username, c2 );
		logon_panel.add( new Label( "Password", Label.LEFT ), c1 );
		logon_panel.add( password, c2 );
		setLayout( new GridBagLayout() );
		Label title = new Label( "POP3 e-mail reader" );
		title.setFont( new Font( "Courier", Font.BOLD | Font.CENTER_BASELINE, 16 ) );
		title.setForeground( new Color( 0, 0, 0xFF ) );
		c2.anchor = GridBagConstraints.CENTER;
		add( title, c2 );
		add( logon_panel, c1 );
		add( connect, c2 );
		c1.fill = GridBagConstraints.BOTH;
		c1.gridheight = 3;
//		c1.gridheight = 4;
		message_panel.add( message_text, c1 );
		c2.fill = GridBagConstraints.BOTH;
		c2.anchor = GridBagConstraints.NORTHWEST;
		message_panel.add( message_list, c2 );
		message_panel.add( message_verbose, c2 );
		message_panel.add( message_delete, c2 );
//		message_panel.add( message_refresh, c2 );
		add( message_panel, c2 );
		add( status, c2 );

		setSize( getPreferredSize() );
	}
	public void start()
	{
	}
	public void stop()
	{
		if( connected ) disconnect();
	}
	public void destroy()
	{
	}

	public int parseMsgNum( String s )
	{
		return Integer.parseInt( s.substring( 0, s.indexOf( " " ) ) );
	}
	public void actionPerformed( ActionEvent e )
	{
		if( e.getSource() == connect )
		{
			if( connected ) disconnect();
			else connect();
		}
		if( e.getSource() == message_delete ) deleteMsg( parseMsgNum( message_list.getSelectedItem() ) );
	}
	public void itemStateChanged( ItemEvent e )
	{
		if( e.getSource() == message_list && e.getStateChange() == ItemEvent.SELECTED )
			displayMsg( parseMsgNum( message_list.getSelectedItem() ) );
		if( e.getSource() == message_verbose ) loadMsgList();
	}

	public void setStatus( boolean error, String msg )
	{
		status.setForeground( error ? new Color( 0x80, 0, 0 ) : new Color( 0, 0, 0x80 ) );
		status.setText( msg );
	}
	public boolean testResult( String s )
	{
		return s.trim().toUpperCase().startsWith( "+OK" );
	}
	public synchronized String sendCommand( String cmd, String comment ) throws IOException, CmdFailedException
	{
		if( comment != null ) setStatus( false, comment );
		output.println( cmd );
		String s = input.readLine();
		if( s == null ) throw new IOException();
		if( !testResult( s ) )
		{
			setStatus( true, "Failed" );
			throw new CmdFailedException();
		}
		return s.substring( 4 );
	}
	public synchronized void connect()
	{
		try
		{
			String proxy_address = getCodeBase().getHost();
			if( proxy_address.length() == 0 ) proxy_address = "localhost";
			setStatus( false, "Connecting..." );
			socket = new Socket( proxy_address, PROXY_PORT );
			if( socket == null ) new IOException();
			output = new PrintStream( socket.getOutputStream() );
			output.println( server.getText() + ": " + POP3_PORT );
			input = new BufferedReader( new InputStreamReader( socket.getInputStream() ) );
			if( !testResult( input.readLine() ) ) throw new IOException();
			try
			{
				sendCommand( "USER " + username.getText(), "Auditentication..." );
				sendCommand( "PASS " + password.getText(), null );
			}
			catch( CmdFailedException e )
			{
				disconnect( "Auditentication failed" );
				return;
			}
			alive = new KeepAlive( this, input, output, KEEPALIVE_INTERVAL );
			alive.start();
		}
		catch( Exception e )
		{
			setStatus( true, "Connection failed" );
			try
			{
				socket.close();
			}
			catch( Exception e1 ) { }
			return;
		}
		setStatus( false, "Connected" );
		connect.setLabel( "Disconnect" );
		connected = true;
		logon_panel.setEnabled( false );
		message_panel.setEnabled( true );
		loadMsgList();
	}
	public synchronized void disconnect()
	{
		connected = false;
		message_verbose.setState( false );
		message_panel.setEnabled( false );
		logon_panel.setEnabled( true );
		message_text.setText( "" );
		message_list.removeAll();
		try
		{
			if( alive != null && alive.isAlive() ) alive.terminate();
			sendCommand( "QUIT", "Disconnecting..." );
		}
		catch( Exception e ) { }
		try
		{
			socket.close();
		}
		catch( Exception e ) { }
		connect.setLabel( "Check mail" );
		setStatus( false, "Not connected" );
	}
	public void disconnect( String errmsg )
	{
		disconnect();
		setStatus( true, errmsg );
	}

	public synchronized String loadMsgHeader( int msgnum ) throws IOException, CmdFailedException
	{
		sendCommand( "TOP " + msgnum + " 0", null );
		String status = "", from = "", subject = "", date = "";
		for( ; ; )
		{
			String s = input.readLine();
			if( s.equals( "." ) ) break;
			int i = s.indexOf( " " );
			if( i < 0 ) continue;
			String s1 = s.substring( 0, i ).toLowerCase();
			String s2 = s.substring( i + 1 );
			if( s1.equals( "status:" ) ) status = s2;
			else if( s1.equals( "from:" ) ) from = s2;
			else if( s1.equals( "subject:" ) ) subject = s2;
			else if( s1.equals( "date:" ) ) date = s2;
		}
		status = ( status.toLowerCase().startsWith( "u" ) )? "(new) " : "";
		return msgnum + " " + status + subject + " (" + from + "; " + date + ")";
	}
	public synchronized void loadMsgList()
	{
		try
		{
			message_list.removeAll();
			sendCommand( "LIST", "Loading messages list..." );
			setStatus( false, "Loading messages list..." );
			for( ; ; )
			{
				String s = input.readLine();
				if( s.equals( "." ) ) break;
				if( s.charAt( 0 ) != '.' ) message_list.add( s );
			}
			if( message_verbose.getState() )
			{
				for( int i = 0; i < message_list.getItemCount(); i++ )
				{
					setStatus( false, "Verbosing message " + i + " ..." );
					String s = loadMsgHeader( parseMsgNum( message_list.getItem( i ) ) );
					message_list.remove( i );
					message_list.add( s, i );
				}
			}
			setStatus( false, "Done" );
		}
		catch( CmdFailedException e )
		{
			setStatus( true, "Failed" );
		}
		catch( Exception e )
		{
			e.printStackTrace();
			disconnect( "Connection lost" );
		}
	}
	public synchronized void displayMsg( int msgnum )
	{
		if( msgnum < 0 ) return;
		try
		{
			message_text.setText( "" );
			sendCommand( "RETR " + msgnum, "Reading message..." );
			for( ; ; )
			{
				String s = input.readLine();
				if( s.equals( "." ) ) break;
				if( s.length() > 0 && s.charAt( 0 ) == '.' ) s = s.substring( 1 );
				message_text.append( s + "\n" );
			}
			message_text.setCaretPosition( 0 );
			setStatus( false, "Done" );
		}
		catch( CmdFailedException e ) {	}
		catch( Exception e )
		{
			disconnect( "Connection lost" );
		}
	}
	public synchronized void deleteMsg( int msgnum )
	{
		if( msgnum < 0 ) return;
		try
		{
			sendCommand( "DELE " + msgnum, "Deleting message..." );
		}
		catch( CmdFailedException e ) {	}
		catch( Exception e )
		{
			disconnect( "Connection lost" );
		}
		loadMsgList();
	}
}

class KeepAlive extends Thread
{
	private Applet applet;
	private PrintStream out;
	private BufferedReader in;
	private int interval;
	private boolean terminate = false;
	public KeepAlive( Applet applet, BufferedReader in, PrintStream out, int interval ) throws IOException
	{
		this.applet = applet;
		this.interval = interval;
		this.in = in;
		this.out = out;
	}
	public void run()
	{
		for( ; ; )
		{
			try
			{
				sleep( interval );
			}
			catch( InterruptedException e ) { }
			if( terminate ) break;
			try
			{
				synchronized( applet )
				{
					out.println( "STAT" );
					in.readLine();
				}
			}
			catch( Exception e ) { }
		}
	}
	public void terminate()
	{
		terminate = true;
	}
}

class CmdFailedException extends Exception { }